from turbogears.decorator import weak_signature_decorator
import sys

class DburiBuildError(Exception):
    """Report an error in building a dburi string."""

class dburi_handler:
    """
    Store portions of a dburi and provide a helper function to build the dburi string.

    Portions of a dburi string can be specified in the following way:
        dbtype: the database type, (mysql, postgres, ...)
	uname: user name to access the database
	passw: password to access the database
	host: server to connect to
	port: port that the database server is running on
	dbname: database name where relevant tables are stored
	filepath: location of file to be used for db access. Probably only 
                  useful for sqlite databases and postgres socket files
        notrans: set to True to disable transactions.
	(additional keywords): any additional keywords are used as arguments.
	                       If one of the above is stepping on a name you need,
                               prefix it with an underscore.
    
    """
    def __init__(self, **kw):
	self.dbtype = None
	self.uname = None
	self.passw = None
	self.host = None
	self.dbname = None
	self.port = None
	self.filepath = None
	self.notrans = False
	self.__dict__.update(kw)

    def _arguments(self):
	"""
	Return a dictionary of the member variables that should be used as dburi arguments.

	returns None if no arguments are present
	"""
	arguments = self.__dict__
	ret = dict()
	poplist = ['dbtype','uname','passw','host','dbname','port','filepath','notrans']
	for (key, value) in arguments:
	    if not key in poplist:
		ret[key.replace('_','',1)] = value
	if len(ret) == 0:
	    return None
	else:
	    return ret

    def build_dburi(self):
	"""
	Attempt to build a useful dburi from the current db information.

	Throws DburiBuildError on failure.
	"""
	dburi = ""
	if not self.dbtype:
	    raise DburiBuildError('Database type is not specified')
	else:
	    if self.notrans:
		dburi = "notrans_" + dburi
	    dburi = dburi + self.dbtype + "://"
	if self.uname and self.passw:
	    dburi = dburi + self.uname
	    if self.passw:
		dburi = dburi + ':' + self.passw
	    dburi = dburi + '@' + self.host
	    if self.port:
		dburi = dburi + ':' + self.port
	    dburi = dburi + '/' + self.dbname
	    # check for any arguments
	    argdict = self._arguments()
	    if argdict:
		dburi = dburi + '?'
		for (key, value) in argdict.items():
		    dburi = dburi + "%s=%s&" % (key, value)
		dburi = dburi[:-1] #nasty
	elif self.filepath:
	    if sys.platform == 'win32':
		return dburi + "/" + self.filepath.replace('\\','/').replace(':','|')
	    else:
		return dburi + self.filepath
	else:
	    raise DburiBuildError('Need a username and password or file path for database connections')
    
def test_db_connection(ormtype='sqlobject', configobj=None):
    """
    Tests the current db connection information, returns True if we can
    connect successfully.
    
    Defaults to sqlobject, set ormtype='sqlalchemy' to test that
    
    Examines the active config by default, set configobj argument to your
    ConfigObj instance to use that instead.
    """
    import turbogears.config
    res = None
    if not configobj:
	getfunc = turbogears.config.get
    else:
	getfunc = configobj.get


    if ormtype == 'sqlobject':
	import sqlobject
	try:
	    uri = getfunc("sqlobject.dburi",'')
	    connection = sqlobject.connectionForURI(uri)
	    res = connection.makeConnection()
	except:
	    return False
    else:
	# I don't use sqlalchemy, so this is a best guess
	import sqlalchemy
	try:
	    uri = getfunc("sqlalchemy.dburi",'')
	    engine = sqlalchemy.create_engine(uri)
	    res = engine.connect()
	except:
	    return False
    if res:
	return True
    else:
	return False
			    
def alter_output(var_func, check_func=None):
    """
    Decorate a controller with output modification functionality. Useful
    for reordering sql results or any other basic data modifications.

    Arguments:
    var_func - a function that accepts the output dictionary, makes any
    appropriate changes to it, and returns it

    check_func - a function that examines the output dictionary and returns
    True if the output should be passed to var_func. If this is not specified
    var_func is automatically called.

    Sample Usage:
    This sample looks at the output for a 'users' key and orders it by
    name, descending only if the users key exists and the functions
    output has an 'orderby' key that is set to 'users'

    def check_descend_user(output):
        try:
            users = output['users']
            orderby = output['orderby']
            if orderby == 'users':
                return True
            else:
                return False
        except keyError:
            return False

    def descend_user(output):
        output['users'] = output[users].orderBy('-name')
        return output

    @alter_output(descend_user, check_descend_user)
    @expose(template="test.templates.viewuser")
    def viewuser(self):
        return(users=model.Users.select(),orderby='users')
    """
    def entangle(func):
        def decorated(func, *args, **kw):
            output = func(*args, **kw)
            if not isinstance(output, dict):
                return output
            if check_func:
                if check_func(output):
                    output = var_func(output)
            else:
                output = var_func(output)
            return output
        return decorated
    return weak_signature_decorator(entangle)
